PGSQL_CONTROL
-- $Id: aggregatex.m4,v 1.12 2006/03/13 10:11:37 aga Exp $

--  
CREATE AGGREGATE cat
     ( BASETYPE = text
     , SFUNC = textcat
     , STYPE = text
     , INITCOND = ''
     )
;
COMMENT ON AGGREGATE cat(text)           IS ' ';



--  bool  AND  OR
CREATE OR REPLACE FUNCTION bool_and(bool, bool) RETURNS bool
 AS 'SELECT $1 AND $2
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION bool_or(bool, bool) RETURNS bool
 AS 'SELECT $1 OR $2
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION bool_and(bool, bool) IS '        ';
COMMENT ON FUNCTION bool_or(bool, bool)  IS '        ';

CREATE AGGREGATE conjunction
     ( BASETYPE = bool
     , SFUNC = bool_and
     , STYPE = bool
     , INITCOND = 1
     )
;
CREATE AGGREGATE disjunction
     ( BASETYPE = bool
     , SFUNC = bool_or
     , STYPE = bool
     , INITCOND = 0
     )
;
COMMENT ON AGGREGATE conjunction(bool)   IS ' AND  bool';
COMMENT ON AGGREGATE disjunction(bool)   IS ' OR  bool';



--  varbit  bool  AND  OR
CREATE OR REPLACE FUNCTION boolor(varbit) RETURNS bool
 AS 'SELECT $1 <> pg_catalog.bit(''0''::varbit, bit_length($1), TRUE)
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION booland(varbit) RETURNS bool
 AS 'SELECT NOT(boolor(bitnot($1)))
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION boolor(varbit)       IS 'OR    ';
COMMENT ON FUNCTION booland(varbit)      IS 'AND    ';



--  varbit  AND  OR
CREATE OR REPLACE FUNCTION bitex0(bit, int4) RETURNS varbit
 AS 'SELECT pg_catalog.bit($1, $2, TRUE)
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION bitex1(bit, int4) RETURNS varbit
 AS 'SELECT CASE WHEN bit_length($1) > $2
                 THEN pg_catalog.bit($1, $2, TRUE)
                 WHEN bit_length($1) < $2
                 THEN bitcat($1, bitnot(pg_catalog.bit(''0''::varbit, $2 - bit_length($1), TRUE)))
                 ELSE $1
            END
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION bitex0(bit, int4) IS '   ,    0';
COMMENT ON FUNCTION bitex1(bit, int4) IS '   ,    1';

CREATE OR REPLACE FUNCTION varbitand(bit, bit) RETURNS varbit
 AS 'SELECT bitand( CASE WHEN d THEN $1 ELSE bitex0($1, l) END
                  , CASE WHEN d THEN bitex0($2, l) ELSE $2 END
                  )
      FROM (SELECT CASE WHEN d THEN bit_length($1) ELSE bit_length($2) END AS l, d
             FROM (SELECT CASE WHEN bit_length($1) > bit_length($2) THEN TRUE ELSE FALSE END AS d)i
           )d
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION varbitor(bit, bit) RETURNS varbit
 AS 'SELECT bitor( CASE WHEN d THEN $1 ELSE bitex0($1, l) END
                 , CASE WHEN d THEN bitex0($2, l) ELSE $2 END
                 )
      FROM (SELECT CASE WHEN d THEN bit_length($1) ELSE bit_length($2) END AS l, d
             FROM (SELECT CASE WHEN bit_length($1) > bit_length($2) THEN TRUE ELSE FALSE END AS d)i
           )d
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION varbitxor(bit, bit) RETURNS varbit
 AS 'SELECT bitxor( CASE WHEN d THEN $1 ELSE bitex0($1, l) END
                  , CASE WHEN d THEN bitex0($2, l) ELSE $2 END
                  )
      FROM (SELECT CASE WHEN d THEN bit_length($1) ELSE bit_length($2) END AS l, d
             FROM (SELECT CASE WHEN bit_length($1) > bit_length($2) THEN TRUE ELSE FALSE END AS d)i
           )d
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION varbitand(bit, bit)  IS 'AND      ';
COMMENT ON FUNCTION varbitor(bit, bit)   IS 'OR      ';
COMMENT ON FUNCTION varbitxor(bit, bit)  IS 'XOR      ';

CREATE OPERATOR /|
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitand
     , COMMUTATOR = /|
     )
;
CREATE OPERATOR |/
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitor
     , COMMUTATOR = |/
     )
;
CREATE OPERATOR /|/
     ( LEFTARG = bit
     , RIGHTARG = bit
     , PROCEDURE = varbitxor
     , COMMUTATOR = /|/
     )
;
COMMENT ON OPERATOR /|(bit, bit)         IS 'varbitand(bit, bit)';
COMMENT ON OPERATOR |/(bit, bit)         IS 'varbitor(bit, bit)';
COMMENT ON OPERATOR /|/(bit, bit)        IS 'varbitxor(bit, bit)';

CREATE OR REPLACE FUNCTION varbitandN(bit, bit) RETURNS varbit
 AS 'SELECT CASE WHEN $1 IS NULL THEN $2 WHEN $2 IS NULL THEN $1 ELSE varbitand($1, $2) END
    '
 LANGUAGE SQL IMMUTABLE
;
CREATE OR REPLACE FUNCTION varbitorN(bit, bit) RETURNS varbit
 AS 'SELECT CASE WHEN $1 IS NULL THEN $2 WHEN $2 IS NULL THEN $1 ELSE varbitor($1, $2) END
    '
 LANGUAGE SQL IMMUTABLE
;
COMMENT ON FUNCTION varbitandN(bit, bit) IS 'varbitand  ';
COMMENT ON FUNCTION varbitorN(bit, bit)  IS 'varbitor  ';

CREATE AGGREGATE conjunction
     ( BASETYPE = bit
     , SFUNC = varbitandN
     , STYPE = varbit
     )
;

CREATE AGGREGATE disjunction
     ( BASETYPE = bit
     , SFUNC = varbitorN
     , STYPE = varbit
     )
;
COMMENT ON AGGREGATE conjunction(bit)    IS ' AND  bit';
COMMENT ON AGGREGATE disjunction(bit)    IS ' OR  bit';



-- 
CREATE OR REPLACE FUNCTION num2bit(bit varying, integer) RETURNS bit varying
 AS '$libdir/num2bit.so.VMAJOR'
  , 'num2bit'
 LANGUAGE C IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION num2bit(integer) RETURNS varbit
 AS 'SELECT num2bit(''0''::varbit, $1)
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION bitreset(varbit, int4) RETURNS varbit
 AS 'SELECT $1 /| bitex1(~num2bit(''0''::varbit, $2), bit_length($1))
    '
 LANGUAGE SQL IMMUTABLE STRICT
;
CREATE AGGREGATE num2bit_aggr
    ( BASETYPE = integer
    , SFUNC = num2bit
    , STYPE = bit varying
    , INITCOND = 0
    )
;

CREATE OR REPLACE FUNCTION bit2num(bit varying) RETURNS SETOF int4
 AS '$libdir/num2bit.so.VMAJOR'
  , 'bit2num'
 LANGUAGE C IMMUTABLE STRICT
;
COMMENT ON FUNCTION bit2num(bit varying)       IS '   ,        ,       ';

CREATE OR REPLACE FUNCTION bit2num_aggr(bit varying, text, text) RETURNS text AS '
    DECLARE
        num int;
        num_prev int;
        fl int;
        str text;
        i RECORD;
    BEGIN
        num_prev := 0;
        fl := 0;
        FOR i IN SELECT bit2num($1) LOOP
            num := i.bit2num;
            IF str IS NULL THEN
                str := num;
            ELSIF num = num_prev + 1 AND fl = 0 THEN
                    str := str || $3;
                    fl := 1;
            ELSIF num <> num_prev + 1 AND fl = 1 THEN
                str := str || num_prev || $2 || num;
                fl := 0;
            ELSIF fl = 0 THEN
                 str := str || $2 || num;
            END IF;
            num_prev := num;
        END LOOP;
        IF fl = 0 THEN
            RETURN str;
        ELSE
            RETURN str || num;
        END IF;
    END;
' LANGUAGE 'plpgsql';

COMMENT ON FUNCTION bit2num_aggr(bit varying, text, text) IS '   ,    .    $2,    $3';


